#include <string.h>
#include <stdint.h>

unsigned char *
write_int64(unsigned char *buffer, uint64_t value)
{
	buffer[0] = value >> 56;
	buffer[1] = value >> 48;
	buffer[2] = value >> 40;
	buffer[3] = value >> 32;
	buffer[4] = value >> 24;
	buffer[5] = value >> 16;
	buffer[6] = value >> 8;
	buffer[7] = value;
	return buffer + 8;
}

unsigned char *
write_int32(unsigned char *buffer, uint32_t value)
{
	buffer[0] = value >> 24;
	buffer[1] = value >> 16;
	buffer[2] = value >> 8;
	buffer[3] = value;
	return buffer + 4;
}

unsigned char *
write_int16(unsigned char *buffer, uint16_t value)
{
	buffer[0] = (value >> 8) & 0xff;
	buffer[1] = value & 0xff;
	return buffer + 2;
}

unsigned char *
write_int8(unsigned char *buffer, uint8_t value)
{
	buffer[0] = value & 0xff;
	return buffer + 1;
}

unsigned char *
write_fixedchar(unsigned char *buffer, char *value, int len)
{
	memset(buffer, 0, len);
	unsigned long slen = strlen(value);
	if (slen > len) {
		slen = len;
	}
	memcpy(buffer, value, slen);
	return buffer + len;
}

unsigned char *
write_varchar(unsigned char *buffer, char *value)
{
	unsigned long len = strlen(value);
	buffer = write_int16(buffer, (int) len);
	buffer = write_fixedchar(buffer, value, (int) len);
	return buffer;
}

unsigned char *
read_int64(unsigned char *buffer, uint64_t *value)
{
	long val = (long) (buffer[7]);
	val += ((long) buffer[6]) << 8;
	val += (long) buffer[5] << 16;
	val += (long) buffer[4] << 24;
	val += (long) buffer[3] << 32;
	val += (long) buffer[2] << 40;
	val += (long) buffer[1] << 48;
	val += (long) buffer[0] << 56;

	*(value) = val;
	return buffer + 8;
}

unsigned char *
read_int32(unsigned char *buffer, uint32_t *value)
{
	(*value) = 0;
	(*value) += buffer[3];
	(*value) += (int) buffer[2] << 8;
	(*value) += (int) buffer[1] << 16;
	(*value) += (int) buffer[0] << 24;

	return buffer + 4;
}

unsigned char *
read_int16(unsigned char *buffer, uint16_t *value)
{
	(*value) = 0;
	(*value) += buffer[1];
	(*value) += (int) buffer[0] << 8;

	return buffer + 2;
}

unsigned char *
read_int8(unsigned char *buffer, uint8_t *value)
{
	(*value) = buffer[0] & 0xFF;
	return buffer + 1;
}

unsigned char *
read_fixedchar(unsigned char *buffer, char *value, int len)
{
	memcpy(value, buffer, len);
	value[len] = '\0';
	return buffer + len;
}

unsigned char *
read_varchar(unsigned char *buffer, char *value)
{
	uint16_t len;
	buffer = read_int16(buffer, &len);
	buffer = read_fixedchar(buffer, value, len);
	return buffer;
}