#define _XOPEN_SOURCE 500

#include "parser.h"

void
parse_log_message(int pri, char *msg, ssize_t len, struct logmessage *result)
{
	result->sent = 0;
	result->format = SRC_UNKNOWN;
	strcpy(result->hostname, "-");
	strcpy(result->app, "-");
	strcpy(result->procid, "-");
	strcpy(result->msgid, "-");
	strcpy(result->fields, "-");

	if (msg[0] == '1' && msg[1] == ' ') {
		// RFC5424 v1
		result->format = SRC_RFC5424;
		msg += 2;
		struct tm tm;
		// TODO: parse fractional time and timezone
		char *outptr = strptime(msg, "%Y-%m-%dT%H:%M:%S", &tm);
		if (outptr != NULL) {
			msg = outptr;
			result->sent = mktime(&tm);
		}
		while (msg[0] != ' ' && msg[0] != '\0') {
			msg++;
		}
		if (msg[0] == ' ') {
			msg++;
		}

		char *ptr = result->hostname;
		while (msg[0] != ' ' && msg[0] != '\0') {
			*ptr++ = msg[0];
			msg++;
		}
		*ptr = '\0';

		if (msg[0] == ' ') {
			msg++;
		}
		ptr = result->app;
		while (msg[0] != ' ' && msg[0] != '\0') {
			*ptr++ = msg[0];
			msg++;
		}
		*ptr = '\0';

		if (msg[0] == ' ') {
			msg++;
		}
		ptr = result->procid;
		while (msg[0] != ' ' && msg[0] != '\0') {
			*ptr++ = msg[0];
			msg++;
		}
		*ptr = '\0';

		if (msg[0] == ' ') {
			msg++;
		}
		ptr = result->msgid;
		while (msg[0] != ' ' && msg[0] != '\0') {
			*ptr++ = msg[0];
			msg++;
		}
		*ptr = '\0';

		if (msg[0] == ' ') {
			msg++;
		}
		if (msg[0] == '[') {
			// TODO: convert fields into json
			ptr = result->fields;
			while (msg[0] != ']' && msg[0] != '\0') {
				*ptr++ = msg[0];
				msg++;
			}
			if (msg[0] == ']') {
				msg++;
				*ptr++ = ']';
			}
			*ptr = '\0';
		}
		if (msg[0] == ' ') {
			msg++;
		}
	} else if (len < 16 || msg[3] != ' ' || msg[6] != ' '
		|| msg[9] != ':' || msg[12] != ':' || msg[15] != ' '
		) {
		// Fallback for weird input
		result->sent = 0;
	} else {
		// RFC3164
		result->format = SRC_RFC3164;
		result->sent = 0;
		msg += 16;
		char *app_msg = msg;
		char *ptr = result->app;
		while (app_msg[0] != ' ' && app_msg[0] != '[' && app_msg[0] != ':' && app_msg[0] != '\0') {
			*ptr++ = app_msg[0];
			app_msg++;
		}

		// Only if this is very likely to be the tag, update msg
		if (app_msg[0] == '[') {
			// A pid probably follows, this is also very likely the tag
			while (app_msg[0] != ']' && app_msg[0] != '\0') {
				*ptr++ = app_msg[0];
				app_msg++;
			}
			*ptr++ = ']';
			*ptr++ = '\0';
			msg = ++app_msg;
		} else if (app_msg[0] == ':') {
			// This was likely the tag
			*ptr++ = '\0';
			msg = ++app_msg;
		} else {
			result->app[0] = '-';
		}

		while (msg[0] == ' ' || msg[0] == ':') {
			msg++;
		}
	}

	result->fac = pri >> 3;
	result->sev = pri & 7;
	strcpy(result->msg, msg);
}