import subprocess
import tempfile
import os
import time
import socket
import sqlite3
import argparse

testdata = [
    {
        "line": "Jul  6 15:53:04 /etc/init.d/chronyd[6225]: WARNING: chronyd has already been started",
        "app": "/etc/init.d/chronyd[6225]",
        "message": "WARNING: chronyd has already been started",
    },
    {
        "line": "Jul  6 15:53:15 : Will stop PID 1839 225",
        "app": "",
        "message": "Will stop PID 1839 225",
    },
    {
        "line": "Jul  6 15:57:11 sudo:     user : TTY=pts/0 ; PWD=/home/user ; USER=root ; COMMAND=/usr/bin/htop",
        "app": "sudo",
        "message": "user : TTY=pts/0 ; PWD=/home/user ; USER=root ; COMMAND=/usr/bin/htop",
    },
    {
        "line": "Jul  6 16:49:53 pkexec[10556]: user: Executing command",
        "app": "pkexec[10556]",
        "message": "user: Executing command",
    }
]

parser = argparse.ArgumentParser()
parser.add_argument("executable")
args = parser.parse_args()


def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d


failures = 0
with tempfile.TemporaryDirectory() as tempdir:
    dbfile = os.path.join(tempdir, "syslog.db")
    syslog_socket = os.path.join(tempdir, "syslog.sock")
    process = subprocess.Popen([args.executable, "-f", "-l", syslog_socket, "-d", dbfile])
    time.sleep(2)
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
    sock.connect(syslog_socket)

    for msg in testdata:
        sock.send(f"<13>{msg['line']}".encode())

    time.sleep(2)
    process.terminate()

    con = sqlite3.connect(dbfile)
    con.row_factory = dict_factory
    cur = con.cursor()
    for idx, row in enumerate(cur.execute("SELECT * FROM log WHERE facility = 1")):
        reference = testdata[idx]
        failed = set()
        fields = ["app", "message"]
        for f in fields:
            if row[f] != reference[f]:
                failed.add(f)

        if len(failed) > 0:
            failures += 1
            print("- " + reference['line'])
            for field in failed:
                print(f"  {field} = '{row[field]}', should've been '{reference[field]}'")

if failures > 0:
    print(f"Failed on {failures} lines")
    exit(1)
