#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "parser.h"

#include "greatest.h"

TEST rfc3164_chrony(void)
{
	char msg[] = "Jul  6 15:53:04 /etc/init.d/chronyd[6225]: WARNING: chronyd has already been started";
	struct logmessage parsed = {0};
	parse_log_message(0, msg, strlen(msg), &parsed);

		ASSERT_EQ_FMTm("FORMAT", SRC_RFC3164, parsed.format, "%d");
		ASSERT_EQ_FMTm("SEV", 0, parsed.sev, "%d");
		ASSERT_EQ_FMTm("FAC", 0, parsed.fac, "%d");
		ASSERT_STR_EQm("APP", "/etc/init.d/chronyd[6225]", parsed.app);
		ASSERT_STR_EQm("HOSTNAME", "-", parsed.hostname);
		ASSERT_STR_EQm("PROCID", "-", parsed.procid);
		ASSERT_STR_EQm("MSGID", "-", parsed.msgid);
		ASSERT_STR_EQm("FIELDS", "-", parsed.fields);
		ASSERT_STR_EQm("MSG", "WARNING: chronyd has already been started", parsed.msg);
		PASS();
}

TEST rfc3164_no_app(void)
{
	char msg[] = "Jul  6 15:53:15 : Will stop PID 1839 225";
	struct logmessage parsed = {0};
	parse_log_message(0, msg, strlen(msg), &parsed);

		ASSERT_EQ_FMTm("FORMAT", SRC_RFC3164, parsed.format, "%d");
		ASSERT_EQ_FMTm("SEV", 0, parsed.sev, "%d");
		ASSERT_EQ_FMTm("FAC", 0, parsed.fac, "%d");
		ASSERT_STR_EQm("APP", "", parsed.app);
		ASSERT_STR_EQm("HOSTNAME", "-", parsed.hostname);
		ASSERT_STR_EQm("PROCID", "-", parsed.procid);
		ASSERT_STR_EQm("MSGID", "-", parsed.msgid);
		ASSERT_STR_EQm("FIELDS", "-", parsed.fields);
		ASSERT_STR_EQm("MSG", "Will stop PID 1839 225", parsed.msg);
		PASS();
}

TEST rfc3164_sudo(void)
{
	char msg[] = "Jul  6 15:57:11 sudo:     user : TTY=pts/0 ; PWD=/home/user ; USER=root ; COMMAND=/usr/bin/htop";
	struct logmessage parsed = {0};
	parse_log_message(0, msg, strlen(msg), &parsed);

		ASSERT_EQ_FMTm("FORMAT", SRC_RFC3164, parsed.format, "%d");
		ASSERT_EQ_FMTm("SEV", 0, parsed.sev, "%d");
		ASSERT_EQ_FMTm("FAC", 0, parsed.fac, "%d");
		ASSERT_STR_EQm("APP", "sudo", parsed.app);
		ASSERT_STR_EQm("HOSTNAME", "-", parsed.hostname);
		ASSERT_STR_EQm("PROCID", "-", parsed.procid);
		ASSERT_STR_EQm("MSGID", "-", parsed.msgid);
		ASSERT_STR_EQm("FIELDS", "-", parsed.fields);
		ASSERT_STR_EQm("MSG", "user : TTY=pts/0 ; PWD=/home/user ; USER=root ; COMMAND=/usr/bin/htop", parsed.msg);
		PASS();
}

TEST rfc3164_pkexec(void)
{
	char msg[] = "Jul  6 16:49:53 pkexec[10556]: user: Executing command";
	struct logmessage parsed = {0};
	parse_log_message(0, msg, strlen(msg), &parsed);

		ASSERT_EQ_FMTm("FORMAT", SRC_RFC3164, parsed.format, "%d");
		ASSERT_EQ_FMTm("SEV", 0, parsed.sev, "%d");
		ASSERT_EQ_FMTm("FAC", 0, parsed.fac, "%d");
		ASSERT_STR_EQm("APP", "pkexec[10556]", parsed.app);
		ASSERT_STR_EQm("HOSTNAME", "-", parsed.hostname);
		ASSERT_STR_EQm("PROCID", "-", parsed.procid);
		ASSERT_STR_EQm("MSGID", "-", parsed.msgid);
		ASSERT_STR_EQm("FIELDS", "-", parsed.fields);
		ASSERT_STR_EQm("MSG", "user: Executing command", parsed.msg);
		PASS();
}


/* Add all the definitions that need to be in the test runner's main file. */
GREATEST_MAIN_DEFS();

int
main(int argc, char **argv)
{
	GREATEST_MAIN_BEGIN();
		RUN_TEST(rfc3164_chrony);
		RUN_TEST(rfc3164_no_app);
		RUN_TEST(rfc3164_sudo);
		RUN_TEST(rfc3164_pkexec);

	GREATEST_MAIN_END();        /* display results */
}